//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include <windows.h>
#include <iostream>
#include "GTAPI_FACE2.h"

std::string to_string( int x ) {
	std::string str = "";

  int length = snprintf( NULL, 0, "%d", x );

  if( length <= 0 )
	return str;

  char* buf = new char[length + 1];
  snprintf( buf, length + 1, "%d", x );
  str = buf;
  delete[] buf;
  return str;
}

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;


void __stdcall local_CB_On_RecvConnected(int ch)
{
	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_CONNECTED;
	msg->chanID = ch;

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_CONNECTED, 0, (LPARAM)msg);
}

void __stdcall local_CB_On_RecvOffered(int ch, const char* sCaller, const char* sCallee, const char* sDestAddr, \
		const char* sViaAddr, const char* sFromIP, unsigned short nFromPort)
{
	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_OFFERED;
	msg->chanID = ch;
	msg->args[0] = sCaller;
	msg->args[1] = sCallee;
	msg->args[2] = sDestAddr;
	msg->args[3] = sViaAddr;
	msg->args[4] = sFromIP;
	msg->args[5] = to_string(nFromPort);

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_OFFERED, 0, (LPARAM)msg);

}

void __stdcall local_CB_On_RecvDialing(int ch, const char* sCaller, const char* sCallee)
{
	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_DIALING;
	msg->chanID = ch;
	msg->args[0] = sCaller;
	msg->args[1] = sCallee;

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_DIALING, 0, (LPARAM)msg);
}

void __stdcall local_CB_On_RecvIdle(int ch, int code, const char* desc)
{
  	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_IDLE;
	msg->chanID = ch;
	msg->args[0] = to_string(code);
	msg->args[1] = desc;

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_IDLE, 0, (LPARAM)msg);

}

void __stdcall local_CB_On_RecvRinging(int ch)
{
	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_RINGING;
	msg->chanID = ch;

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_RINGING, 0, (LPARAM)msg);
}

void __stdcall local_CB_On_RecvRegStatus(int user_id, int status, int regtime)
{
	sdkapp_msg* msg = new sdkapp_msg;
	msg->eventID = WM_SDKAPP_REGSTATUS;
	msg->chanID = user_id;
	msg->args[0] = to_string(status);
	msg->args[1] = to_string(regtime);

	//It is safer to post a message to Main Window instead of directly calling here as it is in low level SDK thread
	if(Form1)
		PostMessage(Form1->Handle, WM_SDKAPP_REGSTATUS, 0, (LPARAM)msg);
}

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
	GTAPI_Create_Env();

	GTAPI_SetMainWnd(Handle);

	GTAPI_CFG_SetValue("gtsrv.sip.server.model", "0"); //client phone

    GTAPI_CFG_SetValue("gtsrv.sip.ip.port", "5890");
    GTAPI_CFG_SetValue("gtsrv.sip.rtpstartrange", "8400");
	GTAPI_CFG_SetValue("gtsrv.sip.rtpendrange", "8800");

    GTAPI_CFG_SetValue("gtsrv.sip.boardnum.per.server", "1");
    GTAPI_CFG_SetValue("gtsrv.sip.spannum.per.board", "1");
    GTAPI_CFG_SetValue("gtsrv.sip.channum.per.span", "1");

    GTAPI_CFG_SetValue("gtsrv.channel.inbound", "0-3");
    GTAPI_CFG_SetValue("gtsrv.channel.outbound", "4-15");

	GTAPI_CFG_SetValue("gtsrv.net.port", "8914");

	GTAPI_SetCB_On_RecvConnected(local_CB_On_RecvConnected);
	GTAPI_SetCB_On_RecvOffered(local_CB_On_RecvOffered);
	GTAPI_SetCB_On_RecvDialing(local_CB_On_RecvDialing);
	GTAPI_SetCB_On_RecvIdle(local_CB_On_RecvIdle);
	GTAPI_SetCB_On_RecvRinging(local_CB_On_RecvRinging);
	GTAPI_SetCB_On_RecvRegStatus(local_CB_On_RecvRegStatus);

	if(!GTAPI_StartServer())
	{
		MessageBox(Handle, _T("GTAPI_StartServer Failed! Please check if SIP port is being used by another application!"), _T("Error"), MB_OK);
		//CloseWindow(Handle);
		Close();
	}

	EnableWindow(btnDial->Handle, TRUE);
	SetWindowText(btnDial->Handle, _T("Dial"));

	EnableWindow(btnHangup->Handle, FALSE);

	SetWindowText(Edit1->Handle, _T("8888@sip.pcbest.net"));

	Memo1->SetTextBuf(_T("IDLE"));

}
//---------------------------------------------------------------------------

void __fastcall TForm1::OnSDKAppMessage(TMessage& tmsg)
{
	sdkapp_msg* msg = (sdkapp_msg*)tmsg.LParam;
	if(!msg)
		return;

	switch(msg->eventID)
	{
		case WM_SDKAPP_OFFERED:
		On_RecvOffered(msg->chanID, msg->args[0].c_str(), msg->args[1].c_str(), msg->args[2].c_str(), msg->args[3].c_str(), msg->args[4].c_str(), atoi(msg->args[5].c_str()));
		break;
		case WM_SDKAPP_DIALING:
		On_RecvDialing(msg->chanID, msg->args[0].c_str(), msg->args[1].c_str());
		break;
		case WM_SDKAPP_RINGING:
		On_RecvRinging(msg->chanID);
		break;
		case WM_SDKAPP_CONNECTED:
		On_RecvConnected(msg->chanID);
		break;
		case WM_SDKAPP_IDLE:
		On_RecvIdle(msg->chanID, atoi(msg->args[0].c_str()), msg->args[1].c_str());
		break;
		case WM_SDKAPP_REGSTATUS:
		On_RecvRegStatus(msg->chanID, atoi(msg->args[0].c_str()), atoi(msg->args[1].c_str()));
		break;
	}

	delete msg;
}


void TForm1::On_RecvConnected(int ch)
{
	GTAPI_StopSound();

	EnableWindow(btnDial->Handle, FALSE);
	EnableWindow(btnHangup->Handle, TRUE);

	Memo1->SetTextBuf(_T("Connected"));

	GTAPI_Send_StartDXAudio(0);
}

void TForm1::On_RecvOffered(int ch, const char* sCaller, const char* sCallee, const char* sDestAddr, \
			const char* sViaAddr, const char* sFromIP, unsigned short nFromPort)
{
    GTAPI_StopSound();
    GTAPI_PlayLocalRingSound();

	EnableWindow(btnDial->Handle, TRUE);
	SetWindowText(btnDial->Handle, _T("Answer"));

	EnableWindow(btnHangup->Handle, TRUE);

	Memo1->SetTextBuf(_T("New Call coming in"));
}

void TForm1::On_RecvDialing(int ch, const char* sCaller, const char* sCallee)
{
	EnableWindow(btnDial->Handle, TRUE);
	EnableWindow(btnHangup->Handle, TRUE);

	Memo1->SetTextBuf(_T("Dialing out"));
}

void TForm1::On_RecvIdle(int ch, int code, const char* desc)
{
    GTAPI_StopSound();

	EnableWindow(btnDial->Handle, TRUE);
	SetWindowText(btnDial->Handle, _T("Dial"));

	EnableWindow(btnHangup->Handle, FALSE);

	Memo1->SetTextBuf(_T("IDLE"));
}

void TForm1::On_RecvRinging(int ch)
{
    GTAPI_StopSound();
    GTAPI_PlayRemoteRingSound();

	EnableWindow(btnDial->Handle, TRUE);
	EnableWindow(btnHangup->Handle, TRUE);

	Memo1->SetTextBuf(_T("Remote Side Ringing"));
}

void TForm1::On_RecvRegStatus(int user_id, int status, int regtime)
{
}


//---------------------------------------------------------------------------
void __fastcall TForm1::btnDialClick(TObject *Sender)
{
	//char sTemp1[256];

	//m_btnDial.GetWindowText(sTemp, 128);

	//sprintf(sTemp1, "<sip:%s>", sTemp);

	GTAPI_Send_Make(0, "<sip:8888@pcbest.net>", "");
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
	GTAPI_StopServer();

	GTAPI_Destroy_Env();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnHangupClick(TObject *Sender)
{
	GTAPI_Send_HangUp(0, 0, "");
}
//---------------------------------------------------------------------------
